# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: api/synerex.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime, timedelta
from typing import AsyncGenerator, List, Optional

import betterproto
import grpclib


class GatewayType(betterproto.Enum):
    BIDIRECTIONAL = 0
    WRITE_ONLY = 1
    READ_ONLY = 2


class MsgType(betterproto.Enum):
    DEMAND = 0
    SUPPLY = 1
    TARGET = 2
    MBUS = 3
    MBUSMSG = 4


@dataclass
class Response(betterproto.Message):
    ok: bool = betterproto.bool_field(1)
    err: str = betterproto.string_field(2)


@dataclass
class ConfirmResponse(betterproto.Message):
    ok: bool = betterproto.bool_field(1)
    mbus_id: float = betterproto.fixed64_field(2)
    wait: timedelta = betterproto.message_field(3)
    err: str = betterproto.string_field(4)


@dataclass
class Content(betterproto.Message):
    entity: bytes = betterproto.bytes_field(1)


@dataclass
class Supply(betterproto.Message):
    id: float = betterproto.fixed64_field(1)
    sender_id: float = betterproto.fixed64_field(2)
    target_id: float = betterproto.fixed64_field(3)
    channel_type: int = betterproto.uint32_field(4)
    supply_name: str = betterproto.string_field(5)
    ts: datetime = betterproto.message_field(6)
    arg_json: str = betterproto.string_field(7)
    mbus_id: float = betterproto.fixed64_field(8)
    cdata: "Content" = betterproto.message_field(9)


@dataclass
class Demand(betterproto.Message):
    id: float = betterproto.fixed64_field(1)
    sender_id: float = betterproto.fixed64_field(2)
    target_id: float = betterproto.fixed64_field(3)
    channel_type: int = betterproto.uint32_field(4)
    demand_name: str = betterproto.string_field(5)
    ts: datetime = betterproto.message_field(6)
    arg_json: str = betterproto.string_field(7)
    mbus_id: float = betterproto.fixed64_field(8)
    cdata: "Content" = betterproto.message_field(9)


@dataclass
class Target(betterproto.Message):
    id: float = betterproto.fixed64_field(1)
    sender_id: float = betterproto.fixed64_field(2)
    target_id: float = betterproto.fixed64_field(3)
    channel_type: int = betterproto.uint32_field(4)
    wait: timedelta = betterproto.message_field(5)
    mbus_id: float = betterproto.fixed64_field(6)


@dataclass
class Channel(betterproto.Message):
    client_id: float = betterproto.fixed64_field(1)
    channel_type: int = betterproto.uint32_field(2)
    arg_json: str = betterproto.string_field(3)


@dataclass
class Mbus(betterproto.Message):
    client_id: float = betterproto.fixed64_field(1)
    mbus_id: float = betterproto.fixed64_field(2)
    arg_json: str = betterproto.string_field(3)


@dataclass
class MbusMsg(betterproto.Message):
    msg_id: float = betterproto.fixed64_field(1)
    sender_id: float = betterproto.fixed64_field(2)
    target_id: float = betterproto.fixed64_field(3)
    mbus_id: float = betterproto.fixed64_field(4)
    msg_type: int = betterproto.uint32_field(5)
    msg_info: str = betterproto.string_field(6)
    arg_json: str = betterproto.string_field(7)


@dataclass
class GatewayInfo(betterproto.Message):
    client_id: float = betterproto.fixed64_field(1)
    gateway_type: "GatewayType" = betterproto.enum_field(2)
    channels: List[int] = betterproto.uint32_field(3)


@dataclass
class GatewayMsg(betterproto.Message):
    """Subscribe from Gateway to SynerexServer"""

    src_synerex_id: float = betterproto.fixed64_field(1)
    msg_type: "MsgType" = betterproto.enum_field(2)
    demand: "Demand" = betterproto.message_field(3, group="msg_oneof")
    supply: "Supply" = betterproto.message_field(4, group="msg_oneof")
    target: "Target" = betterproto.message_field(5, group="msg_oneof")
    mbus: "Mbus" = betterproto.message_field(6, group="msg_oneof")
    mbus_msg: "MbusMsg" = betterproto.message_field(7, group="msg_oneof")


@dataclass
class ProviderID(betterproto.Message):
    client_id: float = betterproto.fixed64_field(1)
    arg_json: str = betterproto.string_field(3)


class SynerexStub(betterproto.ServiceStub):
    async def notify_demand(
        self,
        *,
        id: float = 0,
        sender_id: float = 0,
        target_id: float = 0,
        channel_type: int = 0,
        demand_name: str = "",
        ts: Optional[datetime] = None,
        arg_json: str = "",
        mbus_id: float = 0,
        cdata: Optional["Content"] = None,
    ) -> Response:
        request = Demand()
        request.id = id
        request.sender_id = sender_id
        request.target_id = target_id
        request.channel_type = channel_type
        request.demand_name = demand_name
        if ts is not None:
            request.ts = ts
        request.arg_json = arg_json
        request.mbus_id = mbus_id
        if cdata is not None:
            request.cdata = cdata

        return await self._unary_unary("/api.Synerex/NotifyDemand", request, Response,)

    async def notify_supply(
        self,
        *,
        id: float = 0,
        sender_id: float = 0,
        target_id: float = 0,
        channel_type: int = 0,
        supply_name: str = "",
        ts: Optional[datetime] = None,
        arg_json: str = "",
        mbus_id: float = 0,
        cdata: Optional["Content"] = None,
    ) -> Response:
        request = Supply()
        request.id = id
        request.sender_id = sender_id
        request.target_id = target_id
        request.channel_type = channel_type
        request.supply_name = supply_name
        if ts is not None:
            request.ts = ts
        request.arg_json = arg_json
        request.mbus_id = mbus_id
        if cdata is not None:
            request.cdata = cdata

        return await self._unary_unary("/api.Synerex/NotifySupply", request, Response,)

    async def propose_demand(
        self,
        *,
        id: float = 0,
        sender_id: float = 0,
        target_id: float = 0,
        channel_type: int = 0,
        demand_name: str = "",
        ts: Optional[datetime] = None,
        arg_json: str = "",
        mbus_id: float = 0,
        cdata: Optional["Content"] = None,
    ) -> Response:
        request = Demand()
        request.id = id
        request.sender_id = sender_id
        request.target_id = target_id
        request.channel_type = channel_type
        request.demand_name = demand_name
        if ts is not None:
            request.ts = ts
        request.arg_json = arg_json
        request.mbus_id = mbus_id
        if cdata is not None:
            request.cdata = cdata

        return await self._unary_unary("/api.Synerex/ProposeDemand", request, Response,)

    async def propose_supply(
        self,
        *,
        id: float = 0,
        sender_id: float = 0,
        target_id: float = 0,
        channel_type: int = 0,
        supply_name: str = "",
        ts: Optional[datetime] = None,
        arg_json: str = "",
        mbus_id: float = 0,
        cdata: Optional["Content"] = None,
    ) -> Response:
        request = Supply()
        request.id = id
        request.sender_id = sender_id
        request.target_id = target_id
        request.channel_type = channel_type
        request.supply_name = supply_name
        if ts is not None:
            request.ts = ts
        request.arg_json = arg_json
        request.mbus_id = mbus_id
        if cdata is not None:
            request.cdata = cdata

        return await self._unary_unary("/api.Synerex/ProposeSupply", request, Response,)

    async def select_supply(
        self,
        *,
        id: float = 0,
        sender_id: float = 0,
        target_id: float = 0,
        channel_type: int = 0,
        wait: Optional[timedelta] = None,
        mbus_id: float = 0,
    ) -> ConfirmResponse:
        request = Target()
        request.id = id
        request.sender_id = sender_id
        request.target_id = target_id
        request.channel_type = channel_type
        if wait is not None:
            request.wait = wait
        request.mbus_id = mbus_id

        return await self._unary_unary(
            "/api.Synerex/SelectSupply", request, ConfirmResponse,
        )

    async def select_demand(
        self,
        *,
        id: float = 0,
        sender_id: float = 0,
        target_id: float = 0,
        channel_type: int = 0,
        wait: Optional[timedelta] = None,
        mbus_id: float = 0,
    ) -> ConfirmResponse:
        request = Target()
        request.id = id
        request.sender_id = sender_id
        request.target_id = target_id
        request.channel_type = channel_type
        if wait is not None:
            request.wait = wait
        request.mbus_id = mbus_id

        return await self._unary_unary(
            "/api.Synerex/SelectDemand", request, ConfirmResponse,
        )

    async def confirm(
        self,
        *,
        id: float = 0,
        sender_id: float = 0,
        target_id: float = 0,
        channel_type: int = 0,
        wait: Optional[timedelta] = None,
        mbus_id: float = 0,
    ) -> Response:
        request = Target()
        request.id = id
        request.sender_id = sender_id
        request.target_id = target_id
        request.channel_type = channel_type
        if wait is not None:
            request.wait = wait
        request.mbus_id = mbus_id

        return await self._unary_unary("/api.Synerex/Confirm", request, Response,)

    async def subscribe_demand(
        self, *, client_id: float = 0, channel_type: int = 0, arg_json: str = ""
    ) -> AsyncGenerator[Demand, None]:
        request = Channel()
        request.client_id = client_id
        request.channel_type = channel_type
        request.arg_json = arg_json

        async for response in self._unary_stream(
            "/api.Synerex/SubscribeDemand", request, Demand,
        ):
            yield response

    async def subscribe_supply(
        self, *, client_id: float = 0, channel_type: int = 0, arg_json: str = ""
    ) -> AsyncGenerator[Supply, None]:
        request = Channel()
        request.client_id = client_id
        request.channel_type = channel_type
        request.arg_json = arg_json

        async for response in self._unary_stream(
            "/api.Synerex/SubscribeSupply", request, Supply,
        ):
            yield response

    async def subscribe_mbus(
        self, *, client_id: float = 0, mbus_id: float = 0, arg_json: str = ""
    ) -> AsyncGenerator[MbusMsg, None]:
        request = Mbus()
        request.client_id = client_id
        request.mbus_id = mbus_id
        request.arg_json = arg_json

        async for response in self._unary_stream(
            "/api.Synerex/SubscribeMbus", request, MbusMsg,
        ):
            yield response

    async def send_msg(
        self,
        *,
        msg_id: float = 0,
        sender_id: float = 0,
        target_id: float = 0,
        mbus_id: float = 0,
        msg_type: int = 0,
        msg_info: str = "",
        arg_json: str = "",
    ) -> Response:
        request = MbusMsg()
        request.msg_id = msg_id
        request.sender_id = sender_id
        request.target_id = target_id
        request.mbus_id = mbus_id
        request.msg_type = msg_type
        request.msg_info = msg_info
        request.arg_json = arg_json

        return await self._unary_unary("/api.Synerex/SendMsg", request, Response,)

    async def close_mbus(
        self, *, client_id: float = 0, mbus_id: float = 0, arg_json: str = ""
    ) -> Response:
        request = Mbus()
        request.client_id = client_id
        request.mbus_id = mbus_id
        request.arg_json = arg_json

        return await self._unary_unary("/api.Synerex/CloseMbus", request, Response,)

    async def subscribe_gateway(
        self,
        *,
        client_id: float = 0,
        gateway_type: "GatewayType" = 0,
        channels: List[int] = [],
    ) -> AsyncGenerator[GatewayMsg, None]:
        request = GatewayInfo()
        request.client_id = client_id
        request.gateway_type = gateway_type
        request.channels = channels

        async for response in self._unary_stream(
            "/api.Synerex/SubscribeGateway", request, GatewayMsg,
        ):
            yield response

    async def forward_to_gateway(
        self,
        *,
        src_synerex_id: float = 0,
        msg_type: "MsgType" = 0,
        demand: Optional["Demand"] = None,
        supply: Optional["Supply"] = None,
        target: Optional["Target"] = None,
        mbus: Optional["Mbus"] = None,
        mbus_msg: Optional["MbusMsg"] = None,
    ) -> Response:
        request = GatewayMsg()
        request.src_synerex_id = src_synerex_id
        request.msg_type = msg_type
        if demand is not None:
            request.demand = demand
        if supply is not None:
            request.supply = supply
        if target is not None:
            request.target = target
        if mbus is not None:
            request.mbus = mbus
        if mbus_msg is not None:
            request.mbus_msg = mbus_msg

        return await self._unary_unary(
            "/api.Synerex/ForwardToGateway", request, Response,
        )

    async def close_demand_channel(
        self, *, client_id: float = 0, channel_type: int = 0, arg_json: str = ""
    ) -> Response:
        request = Channel()
        request.client_id = client_id
        request.channel_type = channel_type
        request.arg_json = arg_json

        return await self._unary_unary(
            "/api.Synerex/CloseDemandChannel", request, Response,
        )

    async def close_supply_channel(
        self, *, client_id: float = 0, channel_type: int = 0, arg_json: str = ""
    ) -> Response:
        request = Channel()
        request.client_id = client_id
        request.channel_type = channel_type
        request.arg_json = arg_json

        return await self._unary_unary(
            "/api.Synerex/CloseSupplyChannel", request, Response,
        )

    async def close_all_channels(
        self, *, client_id: float = 0, arg_json: str = ""
    ) -> Response:
        request = ProviderID()
        request.client_id = client_id
        request.arg_json = arg_json

        return await self._unary_unary(
            "/api.Synerex/CloseAllChannels", request, Response,
        )



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: nodeapi/nodeapi.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime
from typing import List, Optional

import betterproto
import grpclib


class NodeType(betterproto.Enum):
    PROVIDER = 0
    SERVER = 1
    GATEWAY = 2


class KeepAliveCommand(betterproto.Enum):
    NONE = 0
    RECONNECT = 1
    SERVER_CHANGE = 2
    PROVIDER_DISCONNECT = 3


@dataclass
class NodeInfo(betterproto.Message):
    """information for synerex servers and providers, gateways (nodes)"""

    node_name: str = betterproto.string_field(1)
    node_type: "NodeType" = betterproto.enum_field(2)
    server_info: str = betterproto.string_field(3)
    node_pbase_version: str = betterproto.string_field(4)
    with_node_id: int = betterproto.int32_field(5)
    cluster_id: int = betterproto.int32_field(6)
    area_id: str = betterproto.string_field(7)
    channel_types: List[int] = betterproto.uint32_field(8)
    gw_info: str = betterproto.string_field(9)
    # for information for controller
    bin_version: str = betterproto.string_field(10)
    count: int = betterproto.int32_field(11)
    last_alive_time: datetime = betterproto.message_field(12)
    keepalive_arg: str = betterproto.string_field(13)


@dataclass
class NodeID(betterproto.Message):
    node_id: int = betterproto.int32_field(1)
    secret: float = betterproto.fixed64_field(2)
    server_info: str = betterproto.string_field(3)
    keepalive_duration: int = betterproto.int32_field(4)


@dataclass
class ServerStatus(betterproto.Message):
    cpu: float = betterproto.double_field(1)
    memory: float = betterproto.double_field(2)
    msg_count: int = betterproto.uint64_field(3)


@dataclass
class NodeUpdate(betterproto.Message):
    node_id: int = betterproto.int32_field(1)
    secret: float = betterproto.fixed64_field(2)
    update_count: int = betterproto.int32_field(3)
    node_status: int = betterproto.int32_field(4)
    node_arg: str = betterproto.string_field(5)
    status: "ServerStatus" = betterproto.message_field(6)


@dataclass
class Response(betterproto.Message):
    ok: bool = betterproto.bool_field(1)
    command: "KeepAliveCommand" = betterproto.enum_field(2)
    err: str = betterproto.string_field(3)


class NodeStub(betterproto.ServiceStub):
    async def register_node(
        self,
        *,
        node_name: str = "",
        node_type: "NodeType" = 0,
        server_info: str = "",
        node_pbase_version: str = "",
        with_node_id: int = 0,
        cluster_id: int = 0,
        area_id: str = "",
        channel_types: List[int] = [],
        gw_info: str = "",
        bin_version: str = "",
        count: int = 0,
        last_alive_time: Optional[datetime] = None,
        keepalive_arg: str = "",
    ) -> NodeID:
        request = NodeInfo()
        request.node_name = node_name
        request.node_type = node_type
        request.server_info = server_info
        request.node_pbase_version = node_pbase_version
        request.with_node_id = with_node_id
        request.cluster_id = cluster_id
        request.area_id = area_id
        request.channel_types = channel_types
        request.gw_info = gw_info
        request.bin_version = bin_version
        request.count = count
        if last_alive_time is not None:
            request.last_alive_time = last_alive_time
        request.keepalive_arg = keepalive_arg

        return await self._unary_unary("/nodeapi.Node/RegisterNode", request, NodeID,)

    async def query_node(
        self,
        *,
        node_id: int = 0,
        secret: float = 0,
        server_info: str = "",
        keepalive_duration: int = 0,
    ) -> NodeInfo:
        request = NodeID()
        request.node_id = node_id
        request.secret = secret
        request.server_info = server_info
        request.keepalive_duration = keepalive_duration

        return await self._unary_unary("/nodeapi.Node/QueryNode", request, NodeInfo,)

    async def keep_alive(
        self,
        *,
        node_id: int = 0,
        secret: float = 0,
        update_count: int = 0,
        node_status: int = 0,
        node_arg: str = "",
        status: Optional["ServerStatus"] = None,
    ) -> Response:
        request = NodeUpdate()
        request.node_id = node_id
        request.secret = secret
        request.update_count = update_count
        request.node_status = node_status
        request.node_arg = node_arg
        if status is not None:
            request.status = status

        return await self._unary_unary("/nodeapi.Node/KeepAlive", request, Response,)

    async def un_register_node(
        self,
        *,
        node_id: int = 0,
        secret: float = 0,
        server_info: str = "",
        keepalive_duration: int = 0,
    ) -> Response:
        request = NodeID()
        request.node_id = node_id
        request.secret = secret
        request.server_info = server_info
        request.keepalive_duration = keepalive_duration

        return await self._unary_unary(
            "/nodeapi.Node/UnRegisterNode", request, Response,
        )



__all__ = ["sxharmovis"]

from synerex_harmovis.sxharmovis import (
    SxHarmoVIS,
    LineStore,
    BGStore,
    ArcStore,
    MeshStore,
    ScatterStore,
    sendBearing,
    sendPitch,
    sendViewState,
    sendBarGraphs,
    drawArcs,
    drawScatters,
    drawLines,
    drawLinesAx,
    drawArcsAx,
    drawScattersAx,
    sendBearingAx,
    sendPitchAx,
    sendViewStateAx,
    sendBarGraphsAx,
    sendTopTextLabelAx,
    sendTopTextLabel,
    clearArcsAx,
    clearArcs,
    clearScattersAx,
    clearScatters,
    sendHarmoVISAx,
    sendHarmoVIS,
    sendMeshAx,
    sendMesh    
)

import synerex_harmovis.proto.geography as geo






from datetime import datetime 
import asyncio
from grpclib.client import Channel

from .proto import api
from .proto import geography as geo

class SxHarmoVIS:
    ''' SxHarmoVIS: base class for Synerex-HarmoVIS integration
    '''
    def __init__(self, srvaddr='localhost:18000', channel=14, node_id = 100):
        hs = srvaddr.split(':')
        self.grpcChannel = Channel(host=hs[0], port=int(hs[1]))
        self.service = api.SynerexStub(self.grpcChannel)
        self.chan = channel
        self.node_id = node_id

    def close(self):
        self.grpcChannel.close()

    async def notifySupply(self, name, data):
        ts = datetime.now()
        cdata = api.Content(entity = data.SerializeToString())
        return await self.service.notify_supply(
            sender_id = self.node_id,
            channel_type = self.chan,
            supply_name = name,
            ts = ts,
            cdata = cdata
        )


    async def sendViewStateAsync(self, lat, lon, zoom = -1, pitch = -1, duration = 0):
        data = geo.ViewState(lat = lat, lon = lon , zoom= zoom, pitch=pitch, duration = duration) 
        return await self.notifySupply('ViewState',data)

    async def sendPitchAsync(self, pitch, duration  =0):
        data = geo.Pitch(pitch = pitch, duration = duration) 
        return await self.notifySupply('Pitch',data)

    async def sendBearingAsync(self, bearing, duration = 0):
        data = geo.Bearing(bearing = bearing, duration = duration) 
        return await self.notifySupply('Bearing',data)

    async def sendBarGraphsAsync(self, bs):
        mybars =[]
        for b in bs.bars:
            bd = []
            ct = 0
            for d in b["barData"]:
                if type(d) is list:
                    bd0=geo.BarData(value = d[0],color = d[1])
                else:
                    bd0=geo.BarData(value = d, color = bs.barColors[ct])
                    ct = ct + 1
                bd.append(bd0)
            mybars.append(geo.BarGraph(id=b["id"], ts=b["ts"],
                                       color_type= b["colorType"],
                                       shape_type=b["shapeType"],
                                       min = b["min"],
                                       max = b["max"],
                                       color = b["color"],
                                       width = b["width"],
                                       radius = b["radius"],
                                       area_color = b["areaColor"],
                                       text = b["text"],
                                       lat = b["lat"],
                                       lon = b["lon"],
                                       bar_data = bd ))
        data = geo.BarGraphs(bars = mybars)
        return await self.notifySupply('BarGraphs',data)

    async def drawLinesAsync(self, lines):
        mylines =[]
        colors = []
        for l in lines.lines:
            mylines.append(geo.Line(from_=[l[0][1],l[0][0]],to=[l[1][1],l[1][0]]))
            colors.append(l[2])
        data = geo.Lines(lines = mylines, width= lines.width, color = colors)
        return await self.notifySupply('Lines',data)

    async def drawArcsAsync(self, arcstore):
        src =[]
        tgt =[]
        srcCol = []
        tgtCol = []
        tilt = []
        for a in arcstore.arcs:
            src.append( geo.Point(lat=a[0],lon=a[1]))
            tgt.append( geo.Point(lat=a[2],lon=a[3]))
            srcCol.append( a[4])
            tgtCol.append( a[5])
            tilt.append( a[6])
        data = geo.Arcs(srcs = src, tgts = tgt, 
                        src_cols = srcCol, 
                        tgt_cols = tgtCol,
                        tilts = tilt
        )
        return await self.notifySupply('Arcs',data)

    async def drawScattersAsync(self, scstore):
        point =[]
        radius =[]
        fillCol = []
        lineCol = []
        for s in scstore.scs:
            point.append( geo.Point(lat=s[0],lon=s[1]))
            radius.append( s[2])
            fillCol.append( s[3])
            lineCol.append( s[4])
        data = geo.Scatters(points = point,
                        radiuses = radius, 
                        fill_colors = fillCol, 
                        line_colors = lineCol
        )
        return await self.notifySupply('Scatters',data)

    async def sendTopTextLabelAsync(self, label, style=""):
        data = geo.TopTextLabel(
            label = label,
            style = style
        )
        return await self.notifySupply('TopTextLabel',data)
    
    async def sendClearArcsAsync(self):
        data = geo.ClearArcs()
        return await self.notifySupply('ClearArcs',data)

    async def sendClearScattersAsync(self):
        data = geo.ClearScatters()
        return await self.notifySupply('ClearScatters',data)

    async def sendHarmoVISAsync(self,jsonStr):
        data = geo.HarmoVIS(jsonStr)
        return await self.notifySupply('HarmoVIS',data)

    async def sendMeshAsync(self, ms):
        return await self.sendHarmoVISAsync(ms.getJsonStr())


class LineStore:
    def __init__(self):
        self.lines = []
        self.width = 1
    
    def setWidth(self, wid):
        self.width = wid

    def addLine(self,lat0,lon0,lat1,lon1, color = 0x708080):
        self.lines.append([[lat0,lon0],[lat1,lon1], color])

class ArcStore:
    def __init__(self):
        self.arcs = []
    
    def addArc(self,lat0,lon0,lat1,lon1, srcCol = 0xd0c000, tgtCol = 0xb0a000, tilt = 0):
        self.arcs.append([lat0,lon0,lat1,lon1, srcCol, tgtCol, tilt])

class ScatterStore:
    def __init__(self):
        self.scs = []
    
    def addScatter(self,lat,lon,radius, fillCol = 0xd000f0, lineCol = 0x200020):
        self.scs.append([lat,lon,radius, fillCol, lineCol])

class MeshStore:
    def __init__(self):
        self.timestamp = int(datetime.now().timestamp())
        self.id = 1000000
        self.count = 0
        self.meshItems = {}

    def setTimestamp(self, ts):
        self.timestamp = ts

    def setTimestampNow(self):
        self.timestamp = int(datetime.now().timestamp())

    def setMeshID(self, id):
        self.id = id

    def addMeshItem(self,lat,lon,value,color="[0,200,0]", itemId = -1):
        if( itemId < 0):
            itemId = self.count
            self.count += 1
            while (itemId in self.meshItems):
                itemId = self.count
                self.count += 1
        self.meshItems[itemId]=[lat,lon,value,color]

    def updateMeshValue(self,itemId,value,color="[0,200,0]"):
        last = self.meshItems[itemId]
        self.meshItems[itemId]=[last[0],last[1],last[2],last[3]]

    def getItemIds(self):
        return self.meshItems.keys()

    def getJsonStr(self):
        mistr = ""        
        for s in self.meshItems.values():
            mistr = mistr + "{" + ('\"pos\":[{},{}],\"val\":{},\"col\":{}'.format(s[1],s[0],s[2], s[3])) +"},"
        meshData = '\"id\":{},\"timestamp\":{}, \"meshItems\":[{}]'.format(self.id, self.timestamp, mistr[:-1])
        jsonConf = '{\"addMesh\":{'+meshData+'}}'
        return jsonConf

class BGStore:
    def __init__(self):
        self.bars = []
        self.barColors = [0xf00000, 0x00f000, 0x0000f0, 0xe0e000, 0xe000e0, 0x00e0e0]
        self.idbase = 0
        self.colorType = geo.BarColorType.VARCOLOR
        self.shapeType = geo.BarShapeType.HEX
        self.areaColor = 0x0f0f0f
        self.color = 0x000d60
        self.radius = 100
        self.width = 30
        

    def addBarData(self,ts, label, lat,lon,barData):
        self.bars.append({
            'id': self.idbase,
            'ts': ts,
            'colorType': self.colorType,
            'shapeType': self.shapeType,
            'min' : 0,
            'max' : 100,
            'color': self.color,
            'width': self.width,
            'radius': self.radius,
            'areaColor': self.areaColor,
            'text' : label,
            'lat':lat,
            'lon':lon,
            'barData':barData
        })
        self.idbase = self.idbase+1
        return self.idbase-1

    def addBarDataNow(self,label,lat,lon, barData):
        ts = datetime.now()
        return self.addBarData(ts, label, lat,lon, barData)

    def updateBarData(self, id, ts, lat, lon, barData):
        self.bars[id].update({
            'ts': ts,
            'lat':lat,
            'lon':lon,
            'barData':barData
        })

    def updateBarDataNow(self,id, lat,lon, barData):
        ts = datetime.now()
        self.updateBarData(id, ts, lat,lon, barData)

async def sendBearingAx(b, duration = 0):
    srv = SxHarmoVIS()
    res = await srv.sendBearingAsync(b, duration)
    srv.close()
    return res

async def sendPitchAx(p, duration = 0):
    srv = SxHarmoVIS()
    res = await srv.sendPitchAsync(p, duration)
    srv.close()
    return res

async def sendViewStateAx(lat,lon,zoom=-1, pitch=-1, duration = 0):
    srv = SxHarmoVIS()
    res = await srv.sendViewStateAsync(lat,lon,zoom,pitch, duration)
    srv.close()
    return res

async def sendBarGraphsAx(bg):
    srv = SxHarmoVIS()
    res = await srv.sendBarGraphsAsync(bg)
    srv.close()
    return res

async def drawLinesAx(ln):
    srv = SxHarmoVIS()
    res = await srv.drawLinesAsync(ln)
    srv.close()
    return res

async def drawArcsAx(ln):
    srv = SxHarmoVIS()
    res = await srv.drawArcsAsync(ln)
    srv.close()
    return res

async def drawScattersAx(ln):
    srv = SxHarmoVIS()
    res = await srv.drawScattersAsync(ln)
    srv.close()
    return res

async def clearArcsAx():
    srv = SxHarmoVIS()
    res = await srv.sendClearArcsAsync()
    srv.close()
    return res

async def clearScattersAx():
    srv = SxHarmoVIS()
    res = await srv.sendClearScattersAsync()
    srv.close()
    return res

async def sendTopTextLabelAx(label, style=""):
    srv = SxHarmoVIS()
    res = await srv.sendTopTextLabelAsync(label, style)
    srv.close()
    return res

async def sendHarmoVISAx(jsonStr):
    srv = SxHarmoVIS()
    res = await srv.sendHarmoVISAsync(jsonStr)
    srv.close()
    return res

async def sendMeshAx(mesh):
    srv = SxHarmoVIS()
    res = await srv.sendMeshAsync(mesh)
    srv.close()
    return res


def sendBearing(b, duration = 0):
    return asyncio.run(sendBearingAx(b, duration))

def sendPitch(p, duration = 0):
    return asyncio.run(sendPitchAx(p, duration))

def sendViewState(lat,lon,zoom=-1, pitch=-1, duration = 0):
    return asyncio.run(sendViewStateAx(lat,lon,zoom,pitch, duration))

def sendBarGraphs(bg):
    return asyncio.run(sendBarGraphsAx(bg))

def drawLines(ln):
    return asyncio.run(drawLinesAx(ln))

def drawArcs(ln):
    return asyncio.run(drawArcsAx(ln))

def drawScatters(ln):
    return asyncio.run(drawScattersAx(ln))

def sendTopTextLabel(label, style=""):
    return asyncio.run(sendTopTextLabelAx(label,style))

def clearArcs():
    return asyncio.run(clearArcsAx())

def clearScatters():
    return asyncio.run(clearScattersAx())

def sendHarmoVIS(jsonStr):
    return asyncio.run(sendHarmoVISAx(jsonStr))

def sendMesh(mesh):
    return asyncio.run(sendMeshAx(mesh))



import synerex_harmovis as sx

import random
import time

import asyncio


def drawRandomLine(count = 20):
    lines = sx.LineStore()
    for i in range(count):
        lines.addLine(
            lat0 = 34.8592285+0.02*random.random(),
            lon0 =136.816348+0.02*random.random(),
            lat1 = 34.8592285+0.02*random.random(),
            lon1 =136.816348+0.02*random.random(),
            color = 0xf0f0f0
        )
    sx.drawLines(lines)


def drawRandomGraph(n=3, count = 10):
    bars = sx.BGStore()
#    bars.colorType = sx.geo.BarColorType.VARCOLOR  # or FIXCOLOR
#    bars.shapeType = sx.geo.BarShapeType.BOX       # or HEX
    for i in range(count):
        bd = []
        for j in range(n):
            bd.append(int(100*random.random()))
        id = bars.addBarDataNow(
            label = "Sample"+str(i),
            lat = 34.8592285+0.01*random.random(),
            lon =136.816348+0.01*random.random(),
            barData = bd 
        )
    sx.sendBarGraphs(bars)
    return bars


def updateRandomGraph(bgstore):
    for bar in bgstore.bars:
        bd = bar['barData']
        for j in range(len(bd)):
            bd[j][0] = int(bd[j][0]*(0.90+random.random()*0.2))
        bgstore.updateBarDataNow(
            id = bar["id"],
            lat = bar["lat"],
            lon = bar["lon"],
            barData = bd 
        )
    sx.sendBarGraphs(bgstore)

def drawRandomArc(count = 20):
    arcs = sx.ArcStore()
    for i in range(count):
        arcs.addArc(
            lat0 = 34.8592285+0.02*random.random(),
            lon0 =136.816348+0.02*random.random(),
            lat1 = 34.8592285+0.02*random.random(),
            lon1 =136.816348+0.02*random.random(),
        )
    sx.drawArcs(arcs)


def main0():   #graph demo
    bars = drawRandomGraph(3,10)
    for i in range(20):
        time.sleep(1)
        updateRandomGraph(bars)

def main1():   #pitch demo
    drawRandomLine()

def main2():
#    loop = asyncio.get_event_loop()
#    loop.run_until_complete(sendBearing(0))
    b =0
    p = 0
    for i in range(72):
        sx.sendBearing(b)
        sx.sendPitch(p)
        p = (p % 60 )+ 3
        b +=5
        time.sleep(0.1)
        sx.send
    

if __name__ == "__main__":
    print("Start Graphdemo for Synerex-Harmovis")
    main0()

    main1()

    main2()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proto/geography/geography.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime
from typing import List

import betterproto


class BarColorType(betterproto.Enum):
    """for BarGraph type"""

    FIXCOLOR = 0
    VARCOLOR = 1


class BarShapeType(betterproto.Enum):
    BOX = 0
    HEX = 1


@dataclass
class Geo(betterproto.Message):
    """geographic data  (mainly for supply)"""

    type: str = betterproto.string_field(1)
    id: int = betterproto.int32_field(2)
    label: str = betterproto.string_field(3)
    data: bytes = betterproto.bytes_field(4)
    options: str = betterproto.string_field(5)


@dataclass
class Line(betterproto.Message):
    from_: List[float] = betterproto.double_field(1)
    to: List[float] = betterproto.double_field(2)


@dataclass
class Lines(betterproto.Message):
    lines: List["Line"] = betterproto.message_field(1)
    width: int = betterproto.int32_field(2)
    color: List[int] = betterproto.int32_field(3)


@dataclass
class Point(betterproto.Message):
    lat: float = betterproto.double_field(1)
    lon: float = betterproto.double_field(2)


@dataclass
class Path(betterproto.Message):
    points: List["Point"] = betterproto.message_field(1)
    width: int = betterproto.int32_field(2)
    color: List[int] = betterproto.int32_field(3)


@dataclass
class Paths(betterproto.Message):
    paths: List["Path"] = betterproto.message_field(1)


@dataclass
class Arcs(betterproto.Message):
    srcs: List["Point"] = betterproto.message_field(1)
    tgts: List["Point"] = betterproto.message_field(2)
    src_cols: List[int] = betterproto.int32_field(3)
    tgt_cols: List[int] = betterproto.int32_field(4)
    tilts: List[int] = betterproto.int32_field(5)


@dataclass
class Scatters(betterproto.Message):
    points: List["Point"] = betterproto.message_field(1)
    radiuses: List[int] = betterproto.int32_field(2)
    fill_colors: List[int] = betterproto.int32_field(3)
    line_colors: List[int] = betterproto.int32_field(4)
    line_width: int = betterproto.int32_field(5)
    fill_alpha: int = betterproto.int32_field(6)


@dataclass
class TopTextLabel(betterproto.Message):
    label: str = betterproto.string_field(1)
    style: str = betterproto.string_field(2)


@dataclass
class ViewState(betterproto.Message):
    lat: float = betterproto.double_field(1)
    lon: float = betterproto.double_field(2)
    zoom: float = betterproto.double_field(3)
    pitch: float = betterproto.double_field(4)
    duration: float = betterproto.double_field(5)


@dataclass
class Bearing(betterproto.Message):
    bearing: float = betterproto.double_field(1)
    duration: float = betterproto.double_field(2)


@dataclass
class Pitch(betterproto.Message):
    pitch: float = betterproto.double_field(1)
    duration: float = betterproto.double_field(2)


@dataclass
class ClearMoves(betterproto.Message):
    message: str = betterproto.string_field(1)


@dataclass
class ClearScatters(betterproto.Message):
    message: str = betterproto.string_field(1)


@dataclass
class ClearArcs(betterproto.Message):
    message: str = betterproto.string_field(1)


@dataclass
class BarData(betterproto.Message):
    label: str = betterproto.string_field(1)
    color: int = betterproto.int32_field(3)
    value: float = betterproto.double_field(2)


@dataclass
class BarGraph(betterproto.Message):
    id: int = betterproto.int32_field(1)
    ts: datetime = betterproto.message_field(2)
    shape_type: "BarShapeType" = betterproto.enum_field(3)
    color_type: "BarColorType" = betterproto.enum_field(4)
    color: int = betterproto.int32_field(5)
    lat: float = betterproto.double_field(6)
    lon: float = betterproto.double_field(7)
    width: float = betterproto.double_field(8)
    radius: float = betterproto.double_field(9)
    min: float = betterproto.double_field(10)
    max: float = betterproto.double_field(11)
    area_color: int = betterproto.int32_field(12)
    bar_data: List["BarData"] = betterproto.message_field(13)
    text: str = betterproto.string_field(14)


@dataclass
class BarGraphs(betterproto.Message):
    bars: List["BarGraph"] = betterproto.message_field(1)


@dataclass
class HarmoVIS(betterproto.Message):
    conf_json: str = betterproto.string_field(1)



""" Synerex_HarmoVIS package
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

def _requires_from_file(filename):
    return open(filename).read().splitlines()

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='synerex_harmovis', 
    version='0.5.2',
    description='Python library for Synerex and HarmoViS',  
    long_description=long_description, 
    long_description_content_type='text/markdown', 
    url='https://github.com/synerex/python_Synerex-HarmoVIS',
    author='Nobuo Kawaguchi',
    author_email='kawaguti@synerex.net',
    classifiers=[  
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Scientific/Engineering :: Visualization',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3 :: Only',
    ],

    keywords='visualization synerex harmoware-vis',
    packages=['synerex_harmovis', 'synerex_harmovis.proto', 'synerex_harmovis.examples'],
    python_requires='>=3.7, <4',

    install_requires=_requires_from_file('requirements.txt'),
    
#    extras_require={  # Optional
#        'dev': ['check-manifest'],
#        'test': ['coverage'],
#    },

    # If there are data files included in your packages that need to be
    # installed, specify them here.
 #   package_data={  # Optional
 #       'sample': ['package_data.dat'],
 #   },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/distutils/setupscript.html#installing-additional-files
    #
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
#    data_files=[('my_data', ['data/data_file'])],  # Optional

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # `pip` to create the appropriate form of executable for the target
    # platform.
    #
    # For example, the following would provide a command called `sample` which
    # executes the function `main` from this package when invoked:
#    entry_points={  # Optional
#        'console_scripts': [
#            'graphdemo=graphdemo:main',
#        ],
#    },

    project_urls={  # Optional
        'Bug Reports': 'https://github.com/synerex/python_Synerex-HarmoVIS/issues',
        'Source': 'https://github.com/synerex/python_Synerex-HarmoVIS/',
    },
)
